#include <cstdlib>
#include <cstring>
#include <iostream>

#include "storybook.h"

Storybook::Storybook(FILE* fin) {
  pages = CyoaParse(fin);
}

void Storybook::UpdateStat(const char *key, int rel) {
  auto it = stats.find(key);
  if (it == stats.end()) {
    // Key not present
    stats[key] = 0;
  } else {
    // Key present, add the rel
    stats[key] = it->second + rel;
  }
}

int Storybook::LookupStat(const char *key) {
  auto it = stats.find(key);
  if (it == stats.end()){
    stats[key] = 0;
    return 0;
  } else {
    return it->second;
  }
  return 0;
}

void Storybook::Find(const char *id) {
  for (Page* page : pages) {
    if (strcmp(page->id, id) == 0) {
      current = page;
      return;
    }
  }
  // If not found
  current = NULL;
}

void Storybook::Advance() {
  // TODO throw an exception if the ending type is choices
  switch(current->footer.type) {
  case FooterType::End:
    IsEnded = true;
    current = NULL;
    return;
  case FooterType::Goto:
    Find(current->footer.link);
    return;
  }
}

void Storybook::Advance(int choice_key) {
  // TODO throw an exception if the ending type is not choices.
  Choice* choice = (*current->footer.choices)[choice_key];
  if (!choice) {
    // TODO throw exception?
  }

  if (auto ck = choice->statcheck) {
    int val = LookupStat(ck->stat);
    if (ck->rel == STATCHECK_GT && val < ck->value ||
	ck->rel == STATCHECK_LT && val > ck->value) {
      // TODO throw different exception?
    }
  }

  if (auto cg = choice->statchange) {
    UpdateStat(cg->stat, cg->addend);
  }

  Find(choice->id);
}

void print_choice(Choice* c) {
  printf("%d) %s", c->option, c->flavor);
  if (c->statcheck) {
    printf("(%s %d%c)",
	   c->statcheck->stat,
	   c->statcheck->value,
	   c->statcheck->rel == STATCHECK_GT ? '+' : '-');
  }
  printf("\n");
}

void Storybook::Play() {
  printf(current->body);
  Choice *choice;

  switch (current->footer.type) {
  case FooterType::End:
    printf("The End.");
    Advance();
    break;
  case FooterType::Goto:
    Advance();
    printf("Press ENTER to continue...");
    std::cin.get();
    break;
  case FooterType::Choices:
    for (auto c : *current->footer.choices) {
      print_choice(c.second);
    }
    printf("Make a selection: ");
    int choice_option;
    std::cin >> choice_option;
    Advance(choice_option);
    break;
  default:
    IsEnded = true; // TODO
  }
}
